@@ -1,9 +1,8 @@
-#!/usr/bin/env python
-# -*- coding: utf-8  -*-
+# -*- coding: utf-8 -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2012 Lance Edgar
+#  Copyright © 2010-2015 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -21,18 +20,22 @@
 #  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
 #
 ################################################################################
-
 """
 Authentication & Authorization
 """
 
+from __future__ import unicode_literals
+
+from edbob.util import prettify
+
+from rattail.db import model
+from rattail.db.auth import has_permission
+
 from zope.interface import implementer
 from pyramid.interfaces import IAuthorizationPolicy
 from pyramid.security import Everyone, Authenticated
 
-from .db import Session
-from rattail.db.model import User
-from rattail.db.auth import has_permission
+from tailbone.db import Session
 
 
 @implementer(IAuthorizationPolicy)
@@ -41,7 +44,7 @@ class TailboneAuthorizationPolicy(object):
     def permits(self, context, principals, permission):
         for userid in principals:
             if userid not in (Everyone, Authenticated):
-                user = Session.query(User).get(userid)
+                user = Session.query(model.User).get(userid)
                 if user:
                     return has_permission(Session(), user, permission)
         if Everyone in principals:
@@ -50,3 +53,29 @@ class TailboneAuthorizationPolicy(object):
 
     def principals_allowed_by_permission(self, context, permission):
         raise NotImplementedError
+
+
+def add_permission_group(config, key, label=None):
+    """
+    Add a permission group to the app configuration.
+    """
+    def action():
+        perms = config.get_settings().get('tailbone_permissions', {})
+        group = perms.setdefault(key, {'key': key})
+        group['label'] = label or prettify(key)
+        config.add_settings({'tailbone_permissions': perms})
+    config.action(None, action)
+
+
+def add_permission(config, groupkey, key, label=None):
+    """
+    Add a permission to the app configuration.
+    """
+    def action():
+        perms = config.get_settings().get('tailbone_permissions', {})
+        group = perms.setdefault(groupkey, {'key': groupkey})
+        group.setdefault('label', prettify(groupkey))
+        perm = group.setdefault('perms', {}).setdefault(key, {'key': key})
+        perm['label'] = label or prettify(key)
+        config.add_settings({'tailbone_permissions': perms})
+    config.action(None, action)
