@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2014 Lance Edgar
+#  Copyright © 2010-2015 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,12 +24,21 @@
 FormAlchemy Field Renderers
 """
 
-from tailbone.forms.renderers.common import (
-    AutocompleteFieldRenderer,
-    DateTimeFieldRenderer,
-    EnumFieldRenderer,
-    YesNoFieldRenderer,
-    )
+from .core import CustomFieldRenderer, DateFieldRenderer
 
-from .people import *
-from .products import *
+from .common import (AutocompleteFieldRenderer,
+                     DecimalFieldRenderer, CurrencyFieldRenderer,
+                     DateTimeFieldRenderer,
+                     EnumFieldRenderer, YesNoFieldRenderer)
+
+from .people import (PersonFieldRenderer, PersonFieldLinkRenderer,
+                     CustomerFieldRenderer, CustomerFieldLinkRenderer,
+                     UserFieldRenderer)
+
+from .employees import EmployeeFieldRenderer
+
+from .products import (ProductFieldRenderer, GPCFieldRenderer,
+                       BrandFieldRenderer, VendorFieldRenderer,
+                       PriceFieldRenderer, PriceWithExpirationFieldRenderer)
+
+from .stores import StoreFieldRenderer
