@@ -124,6 +124,46 @@ class EnumFieldRenderer(SelectFieldRenderer):
         return SelectFieldRenderer.render(self, opts, **kwargs)
 
 
+class DecimalFieldRenderer(formalchemy.FieldRenderer):
+    """
+    Sort of generic field renderer for decimal values.  You must provide the
+    number of places after the decimal (scale).  Note that this in turn relies
+    on simple string formatting; the renderer does not attempt any mathematics
+    of its own.
+    """
+
+    def __init__(self, scale):
+        self.scale = scale
+
+    def __call__(self, field):
+        super(DecimalFieldRenderer, self).__init__(field)
+        return self
+
+    def render_readonly(self, **kwargs):
+        value = self.raw_value
+        if value is None:
+            return ''
+        fmt = '{{0:0.{0}f}}'.format(self.scale)
+        return fmt.format(value)
+
+
+class CurrencyFieldRenderer(formalchemy.FieldRenderer):
+    """
+    Sort of generic field renderer for currency values.
+    """
+
+    def __init__(self, field):
+        super(CurrencyFieldRenderer, self).__init__(field)
+
+    def render_readonly(self, **kwargs):
+        value = self.raw_value
+        if value is None:
+            return ''
+        # TODO: Add py2.6-compatible way to include thousands separator
+        # (comma).  Apparently the ',' trick is py2.7+ only.
+        return '$ {0:0.2f}'.format(value)
+
+
 class YesNoFieldRenderer(CheckBoxFieldRenderer):
 
     def render_readonly(self, **kwargs):
