@@ -54,7 +54,7 @@ class PersonFieldLinkRenderer(PersonFieldRenderer):
         if person:
             return tags.link_to(
                 unicode(person),
-                self.request.route_url('person.read', uuid=person.uuid))
+                self.request.route_url('people.view', uuid=person.uuid))
         return u''
 
 
@@ -76,7 +76,7 @@ class CustomerFieldLinkRenderer(CustomerFieldRenderer):
         if customer:
             return tags.link_to(
                 unicode(customer),
-                self.request.route_url('customer.read', uuid=customer.uuid))
+                self.request.route_url('customers.view', uuid=customer.uuid))
         return u''
 
 
