@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2014 Lance Edgar
+#  Copyright © 2010-2015 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -34,11 +34,6 @@ from webhelpers.html import literal
 from tailbone.util import pretty_datetime
 
 
-__all__ = ['ProductFieldRenderer', 'GPCFieldRenderer',
-           'BrandFieldRenderer', 'VendorFieldRenderer',
-           'PriceFieldRenderer', 'PriceWithExpirationFieldRenderer']
-
-
 class ProductFieldRenderer(AutocompleteFieldRenderer):
     """
     Renderer for :class:`rattail.db.model.Product` instance fields.
@@ -72,6 +67,39 @@ class GPCFieldRenderer(TextFieldRenderer):
         return '{0}-{1}'.format(gpc[:-1], gpc[-1])
 
 
+class DepartmentFieldRenderer(TextFieldRenderer):
+    """
+    Shows the department number as well as the name.
+    """
+    def render_readonly(self, **kwargs):
+        dept = self.raw_value
+        if dept:
+            return "{0} - {1}".format(dept.number, dept.name)
+        return ""
+
+
+class SubdepartmentFieldRenderer(TextFieldRenderer):
+    """
+    Shows the subdepartment number as well as the name.
+    """
+    def render_readonly(self, **kwargs):
+        sub = self.raw_value
+        if sub:
+            return "{0} - {1}".format(sub.number, sub.name)
+        return ""
+
+
+class CategoryFieldRenderer(TextFieldRenderer):
+    """
+    Shows the category number as well as the name.
+    """
+    def render_readonly(self, **kwargs):
+        cat = self.raw_value
+        if cat:
+            return "{0} - {1}".format(cat.number, cat.name)
+        return ""
+
+
 class BrandFieldRenderer(AutocompleteFieldRenderer):
     """
     Renderer for :class:`rattail.db.model.Brand` instance fields.
@@ -84,9 +112,14 @@ class VendorFieldRenderer(AutocompleteFieldRenderer):
     """
     Renderer for :class:`rattail.db.model.Vendor` instance fields.
     """
-
     service_route = 'vendors.autocomplete'
 
+    def render_readonly(self, **kwargs):
+        vendor = self.raw_value
+        if not vendor:
+            return ''
+        return "{0} - {1}".format(vendor.id, vendor.name)
+
 
 class PriceFieldRenderer(TextFieldRenderer):
     """
