@@ -97,6 +97,8 @@ def context_found(event):
     uuid = authenticated_userid(request)
     if uuid:
         request.user = Session.query(User).get(uuid)
+        if request.user:
+            Session().set_continuum_user(request.user)
 
     def has_perm(perm):
         return has_permission(Session(), request.user, perm)
@@ -115,11 +117,9 @@ def context_found(event):
         if request.session.get('referrer'):
             return request.session.pop('referrer')
         referrer = request.referrer
-        if not referrer or referrer == request.current_route_url():
-            if default:
-                referrer = default
-            else:
-                referrer = request.route_url('home')
+        if (not referrer or referrer == request.current_route_url()
+            or not referrer.startswith(request.host_url)):
+            referrer = default or request.route_url('home')
         return referrer
     request.get_referrer = get_referrer
 
