@@ -143,7 +143,7 @@ class ExecuteBatch(View):
         configure_session(self.request.rattail_config, session)
         batch = session.merge(batch)
 
-        if not batch.execute(progress):
+        if not batch.execute(self.request.rattail_config, progress):
             session.rollback()
             session.close()
             return
