@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2014 Lance Edgar
+#  Copyright © 2010-2015 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -32,16 +32,24 @@ except ImportError:
     inspect = None
     from sqlalchemy.orm import class_mapper
 
+import sqlalchemy as sa
+from sqlalchemy_continuum import transaction_class, version_class
+from sqlalchemy_continuum.utils import is_versioned
+
+from rattail.db import model
+from rattail.db.continuum import count_versions, model_transaction_query
+
 from pyramid.httpexceptions import HTTPFound, HTTPNotFound
 
 from .core import View
 
 from ..forms import AlchemyForm
 from formalchemy import FieldSet
-from ..db import Session
 
 from edbob.util import prettify
 
+from tailbone.db import Session
+
 
 __all__ = ['CrudView']
 
@@ -51,6 +59,7 @@ class CrudView(View):
     readonly = False
     allow_successive_creates = False
     update_cancel_route = None
+    child_version_classes = []
 
     @property
     def mapped_class(self):
@@ -87,13 +96,6 @@ class CrudView(View):
         return self.make_fieldset(model)
 
     def make_form(self, model, form_factory=AlchemyForm, **kwargs):
-        if self.readonly:
-            self.creating = False
-            self.updating = False
-        else:
-            self.creating = model is self.mapped_class
-            self.updating = not self.creating
-
         fieldset = self.fieldset(model)
         kwargs.setdefault('pretty_name', self.pretty_name)
         kwargs.setdefault('action_url', self.request.current_route_url())
@@ -119,21 +121,32 @@ class CrudView(View):
     def form(self, model):
         return self.make_form(model)
 
+    def save_form(self, form):
+        form.save()
+
     def crud(self, model, readonly=False):
 
-        if readonly:
-            self.readonly = True
+        self.readonly = readonly
+        if self.readonly:
+            self.creating = False
+            self.updating = False
+        else:
+            self.creating = model is self.mapped_class
+            self.updating = not self.creating
+
+        result = self.pre_crud(model)
+        if result is not None:
+            return result
 
         form = self.form(model)
-        if readonly:
-            form.readonly = True
+        form.readonly = self.readonly
+        if not self.readonly and self.request.method == 'POST':
 
-        if not form.readonly and self.request.POST:
             if form.validate():
-                form.save()
+                self.save_form(form)
 
                 result = self.post_save(form)
-                if result:
+                if result is not None:
                     return result
 
                 if form.creating:
@@ -145,23 +158,57 @@ class CrudView(View):
                     and self.request.params.get('create_and_continue')):
                     return HTTPFound(location=self.request.current_route_url())
 
-                return HTTPFound(location=self.post_save_url(form))
+                if form.creating:
+                    url = self.post_create_url(form)
+                else:
+                    url = self.post_update_url(form)
+                return HTTPFound(location=url)
 
             self.validation_failed(form)
 
+        result = self.post_crud(model, form)
+        if result is not None:
+            return result
+
         kwargs = self.template_kwargs(form)
         kwargs['form'] = form
         return kwargs
 
+    def pre_crud(self, model):
+        pass
+
+    def post_crud(self, model, form):
+        pass
+
     def template_kwargs(self, form):
+        if not form.creating and is_versioned(self.mapped_class):
+            return {'version_count': self.count_versions()}
         return {}
 
+    def count_versions(self):
+        query = self.transaction_query()
+        return query.count()
+
+    def transaction_query(self, parent_class=None, child_classes=None):
+        uuid = self.request.matchdict['uuid']
+        if parent_class is None:
+            parent_class = self.mapped_class
+        if child_classes is None:
+            child_classes = self.child_version_classes
+        return model_transaction_query(Session, uuid, parent_class, child_classes=child_classes)
+
     def post_save(self, form):
         pass
 
     def post_save_url(self, form):
         return self.home_url
 
+    def post_create_url(self, form):
+        return self.post_save_url(form)
+
+    def post_update_url(self, form):
+        return self.post_save_url(form)
+
     def validation_failed(self, form):
         pass
 
@@ -206,14 +253,25 @@ class CrudView(View):
         return self.crud(model)
 
     def delete(self):
+        """
+        View for deleting a record.  Derived classes shouldn't override this,
+        but see also :meth:`pre_delete()` and :meth:`post_delete()`.
+        """
         model = self.get_model_from_request()
         if not model:
             return HTTPNotFound()
+
+        # Let derived classes prep for (or cancel) deletion.
         result = self.pre_delete(model)
-        if result:
+        if result is not None:
             return result
+
+        # Flush the deletion immediately so that we know it will succeed prior
+        # to setting a flash message etc.
         Session.delete(model)
-        Session.flush() # Don't set flash message if delete fails.
+        Session.flush()
+
+        # Derived classes can do extra things here; set flash and go home.
         self.post_delete(model)
         self.flash_delete(model)
         return HTTPFound(location=self.home_url)
