@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2014 Lance Edgar
+#  Copyright © 2010-2015 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,22 +24,21 @@
 Product Views
 """
 
-from __future__ import unicode_literals
+from __future__ import unicode_literals, absolute_import
 
 import os
 import re
 
-from sqlalchemy import and_
-from sqlalchemy.orm import joinedload, aliased
-
-from webhelpers.html.tags import link_to
+import sqlalchemy as sa
+from sqlalchemy import orm
 
+import formalchemy
 from pyramid.httpexceptions import HTTPFound
 from pyramid.renderers import render_to_response
-
-from . import SearchableAlchemyGridView
+from webhelpers.html import tags
 
 import rattail.labels
+from rattail import enum
 from rattail import sil
 from rattail import batches
 from rattail.threads import Thread
@@ -53,10 +52,13 @@ from rattail.db.api import get_product_by_upc
 from rattail.db.util import configure_session
 from rattail.pod import get_image_url, get_image_path
 
-from ..db import Session
-from ..forms import GPCFieldRenderer, BrandFieldRenderer, PriceFieldRenderer
-from . import CrudView
-from ..progress import SessionProgress
+from tailbone.views import SearchableAlchemyGridView, CrudView, AutocompleteView
+from tailbone.views.continuum import VersionView, version_defaults
+from tailbone.forms import EnumFieldRenderer, DateTimeFieldRenderer
+from tailbone.db import Session
+from tailbone.forms import GPCFieldRenderer, BrandFieldRenderer, PriceFieldRenderer
+from tailbone.forms.renderers import products as forms
+from tailbone.progress import SessionProgress
 
 
 class ProductsGrid(SearchableAlchemyGridView):
@@ -68,15 +70,15 @@ class ProductsGrid(SearchableAlchemyGridView):
     # These aliases enable the grid queries to filter products which may be
     # purchased from *any* vendor, and yet sort by only the "preferred" vendor
     # (since that's what shows up in the grid column).
-    ProductCostAny = aliased(ProductCost)
-    VendorAny = aliased(Vendor)
+    ProductCostAny = orm.aliased(ProductCost)
+    VendorAny = orm.aliased(Vendor)
 
     def join_map(self):
 
         def join_vendor(q):
             q = q.outerjoin(
                 ProductCost,
-                and_(
+                sa.and_(
                     ProductCost.product_uuid == Product.uuid,
                     ProductCost.preference == 1,
                     ))
@@ -120,32 +122,9 @@ class ProductsGrid(SearchableAlchemyGridView):
             }
 
     def filter_map(self):
-
-        def filter_upc():
-
-            def filter_is(q, v):
-                if not v:
-                    return q
-                try:
-                    return q.filter(Product.upc.in_((
-                                GPC(v), GPC(v, calc_check_digit='upc'))))
-                except ValueError:
-                    return q
-
-            def filter_not(q, v):
-                if not v:
-                    return q
-                try:
-                    return q.filter(~Product.upc.in_((
-                                GPC(v), GPC(v, calc_check_digit='upc'))))
-                except ValueError:
-                    return q
-
-            return {'is': filter_is, 'nt': filter_not}
-
         return self.make_filter_map(
             ilike=['description', 'size'],
-            upc=filter_upc(),
+            upc=self.filter_gpc(model.Product.upc),
             brand=self.filter_ilike(Brand.name),
             family=self.filter_ilike(model.Family.name),
             department=self.filter_ilike(Department.name),
@@ -183,17 +162,24 @@ class ProductsGrid(SearchableAlchemyGridView):
 
     def query(self):
         q = self.make_query()
-        q = q.options(joinedload(Product.brand))
-        q = q.options(joinedload(Product.department))
-        q = q.options(joinedload(Product.subdepartment))
-        q = q.options(joinedload(Product.regular_price))
-        q = q.options(joinedload(Product.current_price))
-        q = q.options(joinedload(Product.vendor))
+        if not self.request.has_perm('products.view_deleted'):
+            q = q.filter(model.Product.deleted == False)
+        q = q.options(orm.joinedload(Product.brand))
+        q = q.options(orm.joinedload(Product.department))
+        q = q.options(orm.joinedload(Product.subdepartment))
+        q = q.options(orm.joinedload(Product.regular_price))
+        q = q.options(orm.joinedload(Product.current_price))
+        q = q.options(orm.joinedload(Product.vendor))
         return q
 
     def grid(self):
         def extra_row_class(row, i):
-            return 'not-for-sale' if row.not_for_sale else None
+            cls = []
+            if row.not_for_sale:
+                cls.append('not-for-sale')
+            if row.deleted:
+                cls.append('deleted')
+            return ' '.join(cls) if cls else None
         g = self.make_grid(extra_row_class=extra_row_class)
         g.upc.set(renderer=GPCFieldRenderer)
         g.regular_price.set(renderer=PriceFieldRenderer)
@@ -221,11 +207,13 @@ class ProductsGrid(SearchableAlchemyGridView):
             g.deletable = True
             g.delete_route_name = 'product.delete'
 
-        q = Session.query(LabelProfile)
-        if q.count():
-            def labels(row):
-                return link_to("Print", '#', class_='print-label')
-            g.add_column('labels', "Labels", labels)
+        # Maybe add Print Label column.
+        if self.rattail_config.getbool('tailbone', 'products.print_labels', default=True):
+            q = Session.query(LabelProfile)
+            if q.count():
+                def labels(row):
+                    return tags.link_to("Print", '#', class_='print-label')
+                g.add_column('labels', "Labels", labels)
 
         return g
 
@@ -237,9 +225,16 @@ class ProductsGrid(SearchableAlchemyGridView):
 
 
 class ProductCrud(CrudView):
-
+    """
+    Product CRUD view class.
+    """
     mapped_class = Product
     home_route = 'products'
+    child_version_classes = [
+        (model.ProductCode, 'product_uuid'),
+        (model.ProductCost, 'product_uuid'),
+        (model.ProductPrice, 'product_uuid'),
+        ]
 
     def get_model(self, key):
         model = super(ProductCrud, self).get_model(key)
@@ -254,29 +249,59 @@ class ProductCrud(CrudView):
         fs = self.make_fieldset(model)
         fs.upc.set(renderer=GPCFieldRenderer)
         fs.brand.set(options=[])
+        fs.unit_of_measure.set(renderer=EnumFieldRenderer(enum.UNIT_OF_MEASURE))
         fs.regular_price.set(renderer=PriceFieldRenderer)
         fs.current_price.set(renderer=PriceFieldRenderer)
+
+        fs.append(formalchemy.Field('current_price_ends'))
+        fs.current_price_ends.set(value=lambda p: p.current_price.ends if p.current_price else None)
+        fs.current_price_ends.set(renderer=DateTimeFieldRenderer(self.request.rattail_config))
+
+        fs.last_sold.set(renderer=DateTimeFieldRenderer(self.request.rattail_config))
         fs.configure(
             include=[
                 fs.upc.label("UPC"),
                 fs.brand.with_renderer(BrandFieldRenderer),
                 fs.description,
+                fs.unit_size,
+                fs.unit_of_measure.label("Unit of Measure"),
                 fs.size,
-                fs.department,
-                fs.subdepartment,
+                fs.weighed,
+                fs.case_pack,
+                fs.department.with_renderer(forms.DepartmentFieldRenderer),
+                fs.subdepartment.with_renderer(forms.SubdepartmentFieldRenderer),
+                fs.category.with_renderer(forms.CategoryFieldRenderer),
                 fs.family,
                 fs.report_code,
                 fs.regular_price,
                 fs.current_price,
+                fs.current_price_ends,
+                fs.deposit_link,
+                fs.tax,
+                fs.organic,
+                fs.discountable,
+                fs.special_order,
                 fs.not_for_sale,
+                fs.deleted,
+                fs.last_sold,
                 ])
         if not self.readonly:
             del fs.regular_price
             del fs.current_price
+        if not self.request.has_perm('products.view_deleted'):
+            del fs.deleted
         return fs
 
+    def pre_crud(self, product):
+        self.product_deleted = not self.creating and product.deleted
+
+    def post_crud(self, product, form):
+        if self.product_deleted:
+            self.request.session.flash("This product is marked as deleted.", 'error')
+
     def template_kwargs(self, form):
-        kwargs = {'image': False}
+        kwargs = super(ProductCrud, self).template_kwargs(form)
+        kwargs['image'] = False
         product = form.fieldset.model
         if product.upc:
             kwargs['image_url'] = get_image_url(
@@ -288,6 +313,59 @@ class ProductCrud(CrudView):
         return kwargs
 
 
+class ProductVersionView(VersionView):
+    """
+    View which shows version history for a product.
+    """
+    parent_class = model.Product
+    route_model_view = 'product.read'
+    child_classes = [
+        (model.ProductCode, 'product_uuid'),
+        (model.ProductCost, 'product_uuid'),
+        (model.ProductPrice, 'product_uuid'),
+        ]
+
+    def warn_if_deleted(self):
+        """
+        Maybe set flash warning if product is marked deleted.
+        """
+        uuid = self.request.matchdict['uuid']
+        product = Session.query(model.Product).get(uuid)
+        assert product, "No product found for UUID: {0}".format(repr(uuid))
+        if product.deleted:
+            self.request.session.flash("This product is marked as deleted.", 'error')
+
+    def list(self):
+        self.warn_if_deleted()
+        return super(ProductVersionView, self).list()
+
+    def details(self):
+        self.warn_if_deleted()
+        return super(ProductVersionView, self).details()
+
+
+class ProductsAutocomplete(AutocompleteView):
+    """
+    Autocomplete view for products.
+    """
+    mapped_class = model.Product
+    fieldname = 'description'
+
+    def query(self, term):
+        q = Session.query(model.Product).outerjoin(model.Brand)
+        q = q.filter(sa.or_(
+                model.Brand.name.ilike('%{0}%'.format(term)),
+                model.Product.description.ilike('%{0}%'.format(term))))
+        if not self.request.has_perm('products.view_deleted'):
+            q = q.filter(model.Product.deleted == False)
+        q = q.order_by(model.Brand.name, model.Product.description)
+        q = q.options(orm.joinedload(model.Product.brand))
+        return q
+
+    def display(self, product):
+        return product.full_description
+
+
 def products_search(request):
     """
     Locate a product(s) by UPC.
@@ -305,10 +383,14 @@ def products_search(request):
             upc = GPC(upc, calc_check_digit='upc')
             product = get_product_by_upc(Session, upc)
         if product:
-            product = {
-                'uuid':                 product.uuid,
-                'full_description':     product.full_description,
-                }
+            if product.deleted and not request.has_perm('products.view_deleted'):
+                product = None
+            else:
+                product = {
+                    'uuid':                     product.uuid,
+                    'upc':                      unicode(product.upc or ''),
+                    'full_description':         product.full_description,
+                    }
     return {'product': product}
 
 
@@ -370,7 +452,7 @@ class CreateProductsBatch(ProductsGrid):
         if self.request.POST:
             provider = self.request.POST.get('provider')
             if provider:
-                provider = batches.get_provider(provider)
+                provider = batches.get_provider(self.request.rattail_config, provider)
                 if provider:
 
                     if self.request.POST.get('params') == 'True':
@@ -409,6 +491,7 @@ class CreateProductsBatch(ProductsGrid):
 
 def add_routes(config):
     config.add_route('products', '/products')
+    config.add_route('products.autocomplete', '/products/autocomplete')
     config.add_route('products.search', '/products/search')
     config.add_route('products.print_labels', '/products/labels')
     config.add_route('products.create_batch', '/products/batch')
@@ -424,6 +507,11 @@ def includeme(config):
     config.add_view(ProductsGrid, route_name='products',
                     renderer='/products/index.mako',
                     permission='products.list')
+
+    config.add_view(ProductsAutocomplete, route_name='products.autocomplete',
+                    renderer='json',
+                    permission='products.list')
+
     config.add_view(print_labels, route_name='products.print_labels',
                     renderer='json', permission='products.print_labels')
     config.add_view(CreateProductsBatch, route_name='products.create_batch',
@@ -442,3 +530,5 @@ def includeme(config):
                     permission='products.delete')
     config.add_view(products_search, route_name='products.search',
                     renderer='json', permission='products.list')
+
+    version_defaults(config, ProductVersionView, 'product')
